@@ -5,23 +5,26 @@
 
 from enum import Enum, unique
 from logging import getLogger
-from os import X_OK, access, getenv, scandir
-from os.path import abspath, basename, dirname, isfile
+from os import X_OK, access, getenv
+from os.path import abspath, dirname, isfile
 from os.path import join as pathjoin
 from os.path import realpath
+from pathlib import Path
 from platform import system
 from re import IGNORECASE
 from re import compile as re_compile
 from re import match as re_match
 from shutil import rmtree
 from subprocess import Popen, check_output
+from sys import executable
+from typing import Any, Dict, Iterator, List, Optional, Pattern, Set, Tuple, Union
+from urllib.request import pathname2url
 
 try:
-    from subprocess import CREATE_NEW_PROCESS_GROUP
+    # pylint: disable=ungrouped-imports
+    from subprocess import CREATE_NEW_PROCESS_GROUP  # type: ignore[attr-defined]
 except ImportError:
     pass
-from sys import executable
-from urllib.request import pathname2url
 
 from psutil import AccessDenied, NoSuchProcess, wait_procs
 
@@ -54,11 +57,11 @@ __all__ = ("Debugger", "FFPuppet", "Reason")
 class Debugger(Enum):
     """Available "debuggers" to run the browser with"""
 
-    NONE = 0
-    GDB = 1
-    PERNOSCO = 2
-    RR = 3
-    VALGRIND = 4
+    NONE: int = 0
+    GDB: int = 1
+    PERNOSCO: int = 2
+    RR: int = 3
+    VALGRIND: int = 4
 
 
 @unique
@@ -66,13 +69,13 @@ class Reason(Enum):
     """Indicates why the browser process was terminated"""
 
     # target crashed, aborted, triggered an assertion failure, etc...
-    ALERT = 0
+    ALERT: int = 0
     # target was closed by call to FFPuppet close() or has not been launched
-    CLOSED = 1
+    CLOSED: int = 1
     # target exited
-    EXITED = 2
+    EXITED: int = 2
     # target was closed by worker thread
-    WORKER = 3
+    WORKER: int = 3
 
 
 class FFPuppet:
@@ -80,10 +83,10 @@ class FFPuppet:
     This includes setting up the environment, collecting logs and some debugger support.
 
     Attributes:
-        debugger (Debugger): Debugger to use.
-        use_profile (str): Path to existing user profile.
-        use_xvfb (bool): Use Xvfb.
-        working_path (str): Path to use as base directory for temporary files.
+        debugger: Debugger to use.
+        use_profile: Path to existing user profile.
+        use_xvfb: Use Xvfb.
+        working_path: Path to use as base directory for temporary files.
     """
 
     LAUNCH_TIMEOUT_MIN = 10  # minimum amount of time to wait for the browser to launch
@@ -106,122 +109,141 @@ class FFPuppet:
 
     def __init__(
         self,
-        debugger=Debugger.NONE,
-        use_profile=None,
-        use_xvfb=False,
-        working_path=None,
+        debugger: Debugger = Debugger.NONE,
+        use_profile: Optional[str] = None,
+        use_xvfb: bool = False,
+        working_path: Optional[str] = None,
     ):
         # tokens used to notify log scanner to kill the browser process
-        self._abort_tokens = set()
-        self._bin_path = None
-        self._checks = list()
+        self._abort_tokens: Set[Pattern[str]] = set()
+        self._bin_path: Optional[str] = None
+        self._checks: List[
+            Union[CheckLogContents, CheckLogSize, CheckMemoryUsage]
+        ] = list()
         self._dbg = debugger
         self._dbg_sanity_check(self._dbg)
         self._launches = 0  # number of successful browser launches
         self._logs = PuppetLogger(base_path=working_path)
-        self._proc = None
+        self._proc: Optional["Popen[bytes]"] = None
         self._profile_template = use_profile  # profile that is used as a template
         self._xvfb = None
         self._working_path = working_path
-        self.profile = None  # path to profile
-        self.reason = Reason.CLOSED
+        self.profile: Optional[str] = None  # path to profile
+        self.reason: Optional[Reason] = Reason.CLOSED
 
         if use_xvfb:
             if not system().startswith("Linux"):
                 self._logs.clean_up(ignore_errors=True)
-                raise EnvironmentError("Xvfb is only supported on Linux")
+                raise OSError("Xvfb is only supported on Linux")
             try:
                 self._xvfb = Xvfb(width=1280, height=1024)
             except NameError:
                 self._logs.clean_up(ignore_errors=True)
-                raise EnvironmentError("Please install xvfbwrapper") from None
+                raise OSError("Please install xvfbwrapper") from None
             self._xvfb.start()
 
-    def __enter__(self):
+    def __enter__(self) -> "FFPuppet":
         return self
 
-    def __exit__(self, *exc):
+    def __exit__(self, *exc: Any) -> None:
         self.clean_up()
 
-    def _crashreports(self, skip_md=False, skip_benign=True):
+    def _benign_sanitizer_report(self, report: Path) -> bool:
+        """Scan file for benign sanitizer reports.
+
+        Args:
+            report: File to scan.
+
+        Returns:
+            True if only benign reports are found otherwise False.
+        """
+        size = self._logs.watching.get(str(report))
+        # skip previously scanned file if it has not been updated
+        if size is not None and size == report.stat().st_size:
+            return True
+
+        try:
+            # WARNING: cannot open files that are already open on Windows
+            with report.open("rb") as log_fp:
+                # NOTE: only add benign single line warnings here
+                # this should not include any fatal errors
+                for line in log_fp:
+                    line = line.rstrip()
+                    # entries to ignores
+                    if not line:
+                        # empty or blank line
+                        continue
+                    if line.endswith(b"==WARNING: Symbolizer buffer too small"):
+                        # frequently emitted by TSan
+                        continue
+                    if (
+                        b"Sanitizer failed to allocate" in line
+                        and b"==WARNING: " in line
+                    ):
+                        # emitted by *SAN_OPTIONS=max_allocation_size_mb
+                        continue
+                    if b"Sanitizer: soft rss limit exhausted" in line:
+                        # emitted by *SAN_OPTIONS=soft_rss_limit_mb
+                        continue
+                    # line looks interesting so stop scanning
+                    break
+                else:
+                    # nothing interesting was found
+                    LOG.debug("benign log has changed '%s'", report)
+                    self._logs.watching[str(report)] = log_fp.tell()
+                    return True
+
+        except OSError:
+            LOG.debug("failed to scan log %r", str(report))
+
+        return False
+
+    def _crashreports(
+        self, skip_md: bool = False, skip_benign: bool = True
+    ) -> Iterator[Path]:
         """Collect crash logs/reports.
 
         Args:
-            skip_md (bool): Do not scan for minidumps.
-            skip_benign (bool): Skip reports that only contain benign non-fatal
+            skip_md: Do not scan for minidumps.
+            skip_benign: Skip reports that only contain benign non-fatal
                                 warnings.
 
         Yields:
-            str: Path to log on the filesystem.
+            Log on the filesystem.
         """
-        assert self._logs is not None
+        assert self._logs.working_path is not None
         try:
-            contents = scandir(self._logs.working_path)
-        except OSError:  # pragma: no cover
-            contents = tuple()
-        for entry in contents:
-            # scan for sanitizer logs
-            if entry.name.startswith(self._logs.PREFIX_SAN):
-                if skip_benign:
-                    size = self._logs.watching.get(entry.path)
-                    # skip previously scanned files that have not been updated
-                    if size is not None and size == entry.stat().st_size:
+            for entry in Path(self._logs.working_path).iterdir():
+                # scan for sanitizer logs
+                if entry.name.startswith(self._logs.PREFIX_SAN):
+                    if skip_benign and self._benign_sanitizer_report(entry):
                         continue
-                    try:
-                        # WARNING: cannot open files that are already open on Windows
-                        with open(entry.path, "rb") as log_fp:
-                            # NOTE: add only benign single line warnings here
-                            # This should not include any fatal errors
-                            for line in log_fp:
-                                line = line.rstrip()
-                                if not line:
-                                    continue
-                                # entries to ignores
-                                if line.endswith(
-                                    b"==WARNING: Symbolizer buffer too small"
-                                ):
-                                    # frequently emitted by TSan
-                                    continue
-                                if (
-                                    b"Sanitizer failed to allocate" in line
-                                    and b"==WARNING: " in line
-                                ):
-                                    # emitted by *SAN_OPTIONS=max_allocation_size_mb
-                                    continue
-                                if b"Sanitizer: soft rss limit exhausted" in line:
-                                    # emitted by *SAN_OPTIONS=soft_rss_limit_mb
-                                    continue
-                                # the file contains something interesting
-                                break
-                            else:
-                                self._logs.watching[entry.path] = log_fp.tell()
-                                LOG.debug("benign log has changed %r", entry.path)
-                                continue
-                    except OSError:
-                        LOG.debug("failed to scan log %r", entry.path)
-                yield entry.path
-            # scan for Valgrind logs
-            elif self._dbg == Debugger.VALGRIND and entry.name.startswith(
-                self._logs.PREFIX_VALGRIND
-            ):
-                if entry.stat().st_size:
-                    yield entry.path
+                    yield entry.resolve()
+                # scan for Valgrind logs
+                elif self._dbg == Debugger.VALGRIND and entry.name.startswith(
+                    self._logs.PREFIX_VALGRIND
+                ):
+                    if entry.stat().st_size:
+                        yield entry.resolve()
+        except OSError:  # pragma: no cover
+            pass
+
         # check for minidumps
         if not skip_md:
             assert self.profile is not None
             try:
-                for entry in scandir(pathjoin(self.profile, "minidumps")):
+                for entry in (Path(self.profile) / "minidumps").iterdir():
                     if ".dmp" in entry.name:
-                        yield entry.path
+                        yield entry.resolve()
             except OSError:  # pragma: no cover
                 pass
 
     @classmethod
-    def _dbg_sanity_check(cls, dbg):
+    def _dbg_sanity_check(cls, dbg: Debugger) -> None:
         """Check requested debugger is supported and available.
 
         Args:
-            dbg (Debugger): Debugger to sanity check.
+            dbg: Debugger to sanity check.
 
         Returns:
             None
@@ -229,35 +251,33 @@ class FFPuppet:
         LOG.debug("checking %s support", dbg)
         if dbg == Debugger.GDB:
             if not system().startswith("Linux"):
-                raise EnvironmentError("GDB is only supported on Linux")
+                raise OSError("GDB is only supported on Linux")
             try:
                 check_output(["gdb", "--version"])
             except OSError:
-                raise EnvironmentError("Please install GDB") from None
+                raise OSError("Please install GDB") from None
         elif dbg in (Debugger.PERNOSCO, Debugger.RR):
             if not system().startswith("Linux"):
-                raise EnvironmentError("rr is only supported on Linux")
+                raise OSError("rr is only supported on Linux")
             try:
                 check_output(["rr", "--version"])
             except OSError:
-                raise EnvironmentError("Please install rr") from None
+                raise OSError("Please install rr") from None
         elif dbg == Debugger.VALGRIND:
             if not system().startswith("Linux"):
-                raise EnvironmentError("Valgrind is only supported on Linux")
+                raise OSError("Valgrind is only supported on Linux")
             try:
                 match = re_match(
                     b"valgrind-(?P<ver>\\d+\\.\\d+)",
                     check_output(["valgrind", "--version"]),
                 )
             except OSError:
-                raise EnvironmentError("Please install Valgrind") from None
+                raise OSError("Please install Valgrind") from None
             if not match or float(match.group("ver")) < cls.VALGRIND_MIN_VERSION:
-                raise EnvironmentError(
-                    "Valgrind >= %0.2f is required" % cls.VALGRIND_MIN_VERSION
-                )
+                raise OSError(f"Valgrind >= {cls.VALGRIND_MIN_VERSION:.2f} is required")
 
     @staticmethod
-    def _terminate(pid, retry_delay=30, start_mode=0):
+    def _terminate(pid: int, retry_delay: int = 30, start_mode: int = 0) -> None:
         """Terminate the process. Each mode (retry pass) is more aggressive.
         At the end of each attempt if there are active processes mode is
         incremented and another pass is performed.
@@ -268,8 +288,8 @@ class FFPuppet:
         - 2: uses process.kill() on all processes.
 
         Args:
-            retry_delay (int): Time in seconds to wait before next attempt.
-            start_mode (int): Initial mode.
+            retry_delay: Time in seconds to wait before next attempt.
+            start_mode: Initial mode.
 
         Returns:
             None
@@ -307,42 +327,42 @@ class FFPuppet:
                     pass
             raise TerminateError("Failed to terminate browser")
 
-    def add_abort_token(self, token):
+    def add_abort_token(self, token: str) -> None:
         """Add a token that when present in the browser log will have the
         browser process terminated.
 
         Args:
-            token (str): Value to search for in the browser logs.
+            token: Value to search for in the browser logs.
 
         Returns:
             None
         """
-        assert token and isinstance(token, str)
+        assert token
         self._abort_tokens.add(re_compile(token))
 
-    def available_logs(self):
+    def available_logs(self) -> List[str]:
         """List of IDs for the currently available logs.
 
         Args:
             None
 
         Returns:
-            list: A list contains log IDs (str).
+            A list contains log IDs (str).
         """
         return list(self._logs.available_logs())
 
-    def build_launch_cmd(self, bin_path, additional_args=None):
+    def build_launch_cmd(
+        self, bin_path: str, additional_args: Optional[List[str]] = None
+    ) -> List[str]:
         """Build a command that can be used to launch the browser.
 
         Args:
-            bin_path (str): Path to the browser binary.
-            additional_args (list(str)): Additional arguments to pass to the browser.
+            bin_path: Path to the browser binary.
+            additional_args: Additional arguments to pass to the browser.
 
         Returns:
-            list: List of arguments that make up the launch command.
+            List of arguments that make up the launch command.
         """
-        assert isinstance(bin_path, str)
-
         # if a python script is passed use 'sys.executable' as the binary
         # this is used by the test framework
         cmd = list()
@@ -353,11 +373,13 @@ class FFPuppet:
             cmd += ["-profile", self.profile]
 
         if additional_args:
-            assert isinstance(additional_args, list)
-            assert all(isinstance(x, str) for x in additional_args)
             cmd.extend(additional_args)
 
         if self._dbg == Debugger.VALGRIND:
+            assert self._logs.working_path is not None
+            valgrind_log_prefix = pathjoin(
+                self._logs.working_path, self._logs.PREFIX_VALGRIND
+            )
             valgrind_cmd = [
                 "valgrind",
                 "-q",
@@ -367,8 +389,7 @@ class FFPuppet:
                 "--fair-sched=try",
                 "--gen-suppressions=all",
                 "--leak-check=no",
-                "--log-file=%s.%%p"
-                % pathjoin(self._logs.working_path, self._logs.PREFIX_VALGRIND),
+                f"--log-file={valgrind_log_prefix}.%p",
                 "--read-inline-info=no",
                 "--show-mismatched-frees=no",
                 "--show-possibly-lost=no",
@@ -382,9 +403,9 @@ class FFPuppet:
             sup_file = getenv("VALGRIND_SUP_PATH")
             if sup_file:
                 if not isfile(sup_file):
-                    raise IOError("Missing Valgrind suppressions %r" % sup_file)
+                    raise OSError(f"Missing Valgrind suppressions {sup_file!r}")
                 LOG.debug("using Valgrind suppressions: %r", sup_file)
-                valgrind_cmd.append("--suppressions=%s" % sup_file)
+                valgrind_cmd.append(f"--suppressions={sup_file}")
 
             cmd = valgrind_cmd + cmd
 
@@ -443,7 +464,7 @@ class FFPuppet:
 
         return cmd
 
-    def clean_up(self):
+    def clean_up(self) -> None:
         """Remove all remaining files created during execution. This will also
         clear some state information and should only be called once the FFPuppet
         object is no longer needed. Using the FFPuppet object after calling
@@ -473,26 +494,30 @@ class FFPuppet:
         # negative 'self._launches' indicates clean_up() has been called
         self._launches = -1
 
-    def clone_log(self, log_id, offset=None, target_file=None):
+    def clone_log(
+        self,
+        log_id: str,
+        offset: Optional[int] = None,
+        target_file: Optional[str] = None,
+    ) -> Optional[str]:
         """Create a copy of the selected browser log.
 
         Args:
-            log_id (str): ID (key) of the log to clone (stderr, stdout... etc).
-            target_file (str): The log contents will be saved to target_file.
-            offset (int): Location to begin reading the file from.
+            log_id: ID (key) of the log to clone (stderr, stdout... etc).
+            offset: Location to begin reading the file from.
+            target_file: The log contents will be saved to target_file.
 
         Returns:
-            str: Name of the file containing the cloned log or None on failure.
+            Name of the file containing the cloned log or None on failure.
         """
         return self._logs.clone_log(log_id, offset=offset, target_file=target_file)
 
-    def close(self, force_close=False):
+    def close(self, force_close: bool = False) -> None:
         """Terminate the browser process(es) and set `self.reason`. The reason
         code indicates how/why the browser process was terminated.
 
         Args:
-            force_close (bool): Do not collect logs, etc, just make sure
-                                everything is closed.
+            force_close: Do not collect logs, etc, just make sure everything is closed.
 
         Returns:
             None
@@ -515,7 +540,7 @@ class FFPuppet:
             while True:
                 LOG.debug("%d crash report(s) found", len(crash_reports))
                 # wait until crash report files are closed
-                report_wait = 300 if self._dbg == Debugger.RR else 90
+                report_wait = 90 if self._dbg == Debugger.NONE else 300
                 if not wait_on_files(procs, crash_reports, timeout=report_wait):
                     LOG.warning("Crash reports still open after %ds", report_wait)
                     break
@@ -536,6 +561,7 @@ class FFPuppet:
         if self.is_running():
             LOG.debug("browser needs to be terminated")
             # when running under a debugger be less aggressive
+            assert pid is not None
             self._terminate(pid, start_mode=1 if self._dbg == Debugger.NONE else 0)
         # wait for any remaining processes to close
         if wait_procs(procs, timeout=1 if force_close else 30)[1]:
@@ -556,23 +582,30 @@ class FFPuppet:
                     if check.message is not None:
                         r_code = Reason.WORKER
                         check.dump_log(
-                            dst_fp=self._logs.add_log("ffp_worker_%s" % check.name)
+                            dst_fp=self._logs.add_log(f"ffp_worker_{check.name}")
                         )
                 # collect logs (excluding minidumps)
-                for fname in self._crashreports(skip_md=True, skip_benign=False):
-                    # pylint: disable=consider-using-with
-                    self._logs.add_log(basename(fname), open(fname, "rb"))
-                # check for minidumps in the profile and dump them if possible
-                process_minidumps(
-                    pathjoin(self.profile, "minidumps"),
-                    pathjoin(self._bin_path, "symbols"),
-                    self._logs.add_log,
-                    working_path=self._working_path,
-                )
-                if self._logs.get_fp("stderr"):
-                    self._logs.get_fp("stderr").write(
-                        ("[ffpuppet] Reason code: %s\n" % r_code.name).encode("utf-8")
+                for log_path in self._crashreports(skip_md=True, skip_benign=False):
+                    self._logs.add_log(log_path.name, log_path.open("rb"))
+                assert self.profile is not None
+                assert self._bin_path is not None
+                # check for minidumps and process them if possible
+                md_path = Path(self.profile) / "minidumps"
+                if any(md_path.glob("*.dmp")):
+                    # check for local build symbols
+                    sym_path = Path(self._bin_path) / ".." / "crashreporter-symbols"
+                    if not sym_path.is_dir():
+                        # use packaged symbols
+                        sym_path = Path(self._bin_path) / "symbols"
+                    process_minidumps(
+                        md_path,
+                        sym_path,
+                        self._logs.add_log,
+                        working_path=self._working_path,
                     )
+                stderr_fp = self._logs.get_fp("stderr")
+                if stderr_fp:
+                    stderr_fp.write(f"[ffpuppet] Reason code: {r_code.name}\n".encode())
         # reset remaining to closed state
         try:
             self._proc = None
@@ -580,6 +613,7 @@ class FFPuppet:
             self._checks = list()
             # remove temporary profile directory if necessary
             try:
+                assert self.profile is not None
                 rmtree(self.profile, onerror=onerror)
             except OSError:  # pragma: no cover
                 LOG.error("Failed to remove profile %r", self.profile)
@@ -591,14 +625,14 @@ class FFPuppet:
             LOG.debug("reason code: %s", r_code.name)
             self.reason = r_code
 
-    def cpu_usage(self):
+    def cpu_usage(self) -> Iterator[Tuple[int, float]]:
         """Collect percentage of CPU usage per process.
 
         Args:
             None
 
         Yields:
-            tuple: PID and the CPU usage as a percentage.
+            PID and the CPU usage as a percentage.
         """
         pid = self.get_pid()
         if pid is not None:
@@ -608,21 +642,21 @@ class FFPuppet:
                 except (AccessDenied, NoSuchProcess):  # pragma: no cover
                     continue
 
-    def get_pid(self):
+    def get_pid(self) -> Optional[int]:
         """Get the browser process ID.
 
         Args:
             None
 
         Returns:
-            int: Browser PID.
+            Browser PID.
         """
         try:
-            return self._proc.pid
+            return self._proc.pid  # type: ignore[union-attr]
         except AttributeError:
             return None
 
-    def is_healthy(self):
+    def is_healthy(self) -> bool:
         """Verify the browser is in a good state by performing a series
         of checks.
 
@@ -630,8 +664,8 @@ class FFPuppet:
             None
 
         Returns:
-            bool: True if the browser is running and determined to be
-                  in a valid functioning state otherwise False.
+            True if the browser is running and determined to be
+            in a valid functioning state otherwise False.
         """
         if self.reason is not None:
             LOG.debug("reason is set to %r", self.reason.name)
@@ -648,52 +682,46 @@ class FFPuppet:
                 return False
         return True
 
-    def is_running(self):
+    def is_running(self) -> bool:
         """Check if the browser process is running.
 
         Args:
             None
 
         Returns:
-            bool: True if the process is running otherwise False.
+            True if the process is running otherwise False.
         """
         try:
-            return self._proc.poll() is None
+            return self._proc.poll() is None  # type: ignore[union-attr]
         except AttributeError:
             return False
 
     def launch(
         self,
-        bin_path,
-        env_mod=None,
-        launch_timeout=300,
-        location=None,
-        log_limit=0,
-        memory_limit=0,
-        prefs_js=None,
-        extension=None,
-    ):
+        bin_path: str,
+        env_mod: Optional[Dict[str, Optional[str]]] = None,
+        launch_timeout: int = 300,
+        location: Optional[str] = None,
+        log_limit: int = 0,
+        memory_limit: int = 0,
+        prefs_js: Optional[str] = None,
+        extension: Optional[str] = None,
+    ) -> None:
         """Launch a new browser process.
 
         Args:
-            bin_path (str): Path to the Firefox binary.
-            env_mod (dict): Environment modifier. Add, remove and update entries
-                            in the prepared environment. Add and update by
-                            setting value (str) and remove by setting entry
-                            value to None.
-            launch_timeout (int): Timeout in seconds for launching the browser.
-            location (str): URL to navigate to after successfully launch of
-                            the browser.
-            log_limit (int): Log file size limit in bytes. Browser will be
-                             terminated if the log file exceeds the amount
-                             specified.
-            memory_limit (int): Memory limit in bytes. Browser will be
-                                terminated if its memory usage exceeds the
-                                amount specified.
-            prefs_js (str): Path to a prefs.js file to install in the Firefox
-                            profile.
-            extension (str): Path to an extension (or list of extension) to be
-                             installed.
+            bin_path: Path to the Firefox binary.
+            env_mod: Environment modifier. Add, remove and update entries
+                     in the prepared environment. Add and update by
+                     setting value (str) and remove by setting entry value to None.
+            launch_timeout: Timeout in seconds for launching the browser.
+            location: URL to navigate to after successfully launch of the browser.
+            log_limit: Log file size limit in bytes. Browser will be
+                       terminated if the log file exceeds the amount specified.
+            memory_limit: Memory limit in bytes. Browser will be terminated
+                          if its memory usage exceeds the amount specified.
+            prefs_js: Path to a prefs.js file to install in the Firefox profile.
+            extension: Path to an extension (or list of extension) to be installed.
 
         Returns:
             None
@@ -706,7 +734,7 @@ class FFPuppet:
 
         bin_path = abspath(bin_path)
         if not isfile(bin_path) or not access(bin_path, X_OK):
-            raise IOError("%s is not an executable" % bin_path)
+            raise OSError(f"{bin_path} is not an executable")
         # need the path to help find symbols
         self._bin_path = dirname(bin_path)
 
@@ -717,7 +745,7 @@ class FFPuppet:
                     ["file:", pathname2url(realpath(location)).lstrip("/")]
                 )
             elif re_match(r"http(s)?://", location, IGNORECASE) is None:
-                raise IOError("Cannot find %r" % location)
+                raise OSError(f"Cannot find {location!r}")
 
         # create and modify a profile
         self.profile = create_profile(
@@ -743,16 +771,12 @@ class FFPuppet:
             # reducing older Grizzly test cases.
             prefs = {
                 "capability.policy.localfilelinks.checkloaduri.enabled": "'allAccess'",
-                "capability.policy.localfilelinks.sites": "'%s'"
-                % bootstrapper.location,
+                "capability.policy.localfilelinks.sites": f"'{bootstrapper.location}'",
                 "capability.policy.policynames": "'localfilelinks'",
                 "network.proxy.allow_bypass": "false",
                 "network.proxy.failover_direct": "false",
                 "privacy.partition.network_state": "false",
             }
-            if self._dbg in (Debugger.PERNOSCO, Debugger.RR, Debugger.VALGRIND):
-                # if the browser is running slowly socket reads can fail if this is > 0
-                prefs["network.http.speculative-parallel-limit"] = "0"
             append_prefs(self.profile, prefs)
 
             launch_args = [bootstrapper.location]
@@ -777,10 +801,9 @@ class FFPuppet:
             # open logs
             self._logs.add_log("stdout")
             stderr = self._logs.add_log("stderr")
-            stderr.write(b"[ffpuppet] Launch command: ")
-            stderr.write(" ".join(cmd).encode("utf-8"))
-            stderr.write(b"\n\n")
+            stderr.write(f"[ffpuppet] Launch command: {' '.join(cmd)}\n\n".encode())
             stderr.flush()
+            assert self._logs.working_path is not None
             sanitizer_logs = pathjoin(self._logs.working_path, self._logs.PREFIX_SAN)
             # launch the browser
             LOG.debug("launch command: %r", " ".join(cmd))
@@ -803,24 +826,30 @@ class FFPuppet:
             if prefs_js is not None and isfile(
                 pathjoin(self.profile, "Invalidprefs.js")
             ):
-                raise InvalidPrefs("%r is invalid" % prefs_js)
+                raise InvalidPrefs(f"{prefs_js!r} is invalid")
 
+        logs_fp_stderr = self._logs.get_fp("stderr")
+        assert logs_fp_stderr is not None
+        logs_fp_stdout = self._logs.get_fp("stdout")
+        assert logs_fp_stdout is not None
         if log_limit:
             self._checks.append(
                 CheckLogSize(
                     log_limit,
-                    self._logs.get_fp("stderr").name,
-                    self._logs.get_fp("stdout").name,
+                    logs_fp_stderr.name,
+                    logs_fp_stdout.name,
                 )
             )
         if memory_limit:
-            self._checks.append(CheckMemoryUsage(self.get_pid(), memory_limit))
+            curr_pid = self.get_pid()
+            assert curr_pid is not None
+            self._checks.append(CheckMemoryUsage(curr_pid, memory_limit))
         if self._abort_tokens:
             self._checks.append(
                 CheckLogContents(
                     [
-                        self._logs.get_fp("stderr").name,
-                        self._logs.get_fp("stdout").name,
+                        logs_fp_stderr.name,
+                        logs_fp_stdout.name,
                     ],
                     self._abort_tokens,
                 )
@@ -829,39 +858,39 @@ class FFPuppet:
         self._launches += 1
 
     @property
-    def launches(self):
+    def launches(self) -> int:
         """Number of successful launches.
 
         Args:
             None
 
         Returns:
-            int: Successful launch count.
+            Successful launch count.
         """
         assert self._launches > -1, "clean_up() has been called"
         return self._launches
 
-    def log_length(self, log_id):
+    def log_length(self, log_id: str) -> Optional[int]:
         """Get the length of the selected browser log.
 
         Args:
-            log_id (str): ID (key) of the log (stderr, stdout... etc).
+            log_id: ID (key) of the log (stderr, stdout... etc).
 
         Returns:
-            int: Length of the log in bytes.
+            Length of the log in bytes.
         """
         return self._logs.log_length(log_id)
 
-    def save_logs(self, dest, logs_only=False, meta=False):
+    def save_logs(self, dest: str, logs_only: bool = False, meta: bool = False) -> None:
         """The browser logs will be saved to dest. This can only be called
         after close().
 
         Args:
-            dest (str): Destination path for log data. Existing files will
+            dest: Destination path for log data. Existing files will
                         be overwritten.
-            logs_only (bool): Do not include other data such as debugger
+            logs_only: Do not include other data such as debugger
                               output files.
-            meta (bool): Output JSON file containing log file meta data.
+            meta: Output JSON file containing log file meta data.
 
         Returns:
             None
@@ -877,17 +906,17 @@ class FFPuppet:
             rr_pack=self._dbg in (Debugger.PERNOSCO, Debugger.RR),
         )
 
-    def wait(self, timeout=None):
+    def wait(self, timeout: Optional[float] = None) -> bool:
         """Wait for browser process(es) to terminate. This call will block until
         all process(es) exit unless a timeout is specified. If a timeout of zero
         or greater is specified the call will block until the timeout expires.
 
         Args:
-            timeout (float): The maximum amount of time in seconds to wait or
+            timeout: The maximum amount of time in seconds to wait or
                              None (wait indefinitely).
 
         Returns:
-            bool: True if processes exit before timeout expires otherwise False.
+            True if processes exit before timeout expires otherwise False.
         """
         assert timeout is None or timeout >= 0
         pid = self.get_pid()
