@@ -3,7 +3,7 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 """ffpuppet main.py"""
 
-from argparse import ArgumentParser
+from argparse import ArgumentParser, Namespace
 from logging import DEBUG, ERROR, INFO, WARNING, basicConfig, getLogger
 from os import scandir
 from os.path import abspath, exists, isdir, isfile
@@ -12,6 +12,7 @@ from platform import system
 from shutil import rmtree
 from tempfile import mkdtemp
 from time import sleep, strftime
+from typing import List, Optional
 
 from .core import Debugger, FFPuppet, Reason
 from .helpers import check_prefs
@@ -21,15 +22,15 @@ LOG = getLogger(__name__)
 __author__ = "Tyson Smith"
 
 
-def dump_to_console(log_dir, log_quota=0x8000):
+def dump_to_console(log_dir: str, log_quota: int = 0x8000) -> str:
     """Read and merge log files and format for output on the console.
 
     Args:
-        log_dir (str): Directory to scan for logs.
-        log_quota (int): Maximum number of bytes to read per log.
+        log_dir: Directory to scan for logs.
+        log_quota: Maximum number of bytes to read per log.
 
     Returns:
-        str: Merged log data to be displayed on the console.
+        Merged log data to be displayed on the console.
     """
 
     logs = list(x for x in scandir(log_dir) if x.is_file())
@@ -52,18 +53,27 @@ def dump_to_console(log_dir, log_quota=0x8000):
     for log in logs:
         fsize = log.stat().st_size
         lines.append("\n===\n")
-        lines.append("=== Dumping %r (%0.2fKB)" % (log.name, fsize / 1024.0))
+        lines.append(f"=== Dumping {log.name!r} ({fsize / 1024.0:0.2f}KB)")
         with open(log.path, "rb") as log_fp:
             # tail log if needed
             log_fp.seek(max(fsize - log_quota, 0))
             if log_fp.tell() > 0:
-                lines.append(" - tailed (%0.2fKB)" % (log_quota / 1024.0))
+                lines.append(f" - tailed ({log_quota / 1024.0:0.2f}KB)")
             lines.append("\n===\n")
             lines.append(log_fp.read().decode("ascii", errors="ignore"))
     return "".join(lines)
 
 
-def parse_args(argv=None):  # pylint: disable=missing-docstring
+def parse_args(argv: Optional[List[str]] = None) -> Namespace:
+    """Handle argument parsing.
+
+    Args:
+        argv: Arguments from the user.
+
+    Returns:
+        Parsed and sanitized arguments.
+    """
+
     log_level_map = {"ERROR": ERROR, "WARN": WARNING, "INFO": INFO, "DEBUG": DEBUG}
 
     parser = ArgumentParser(
@@ -79,9 +89,9 @@ def parse_args(argv=None):  # pylint: disable=missing-docstring
     )
     parser.add_argument(
         "--log-level",
+        choices=sorted(log_level_map),
         default="INFO",
-        help="Configure console logging. Options: %s (default: %%(default)s)"
-        % ", ".join(k for k, v in sorted(log_level_map.items(), key=lambda x: x[1])),
+        help="Configure console logging (default: %(default)s)",
     )
 
     cfg_group = parser.add_argument_group("Browser Configuration")
@@ -192,38 +202,33 @@ def parse_args(argv=None):  # pylint: disable=missing-docstring
             help="Use Valgrind.",
         )
 
-    parser.set_defaults(
-        debugger=Debugger.NONE,
-    )
+    parser.set_defaults(debugger=Debugger.NONE)
 
     args = parser.parse_args(argv)
 
     # sanity checks
     if not isfile(args.binary):
-        parser.error("Invalid browser binary %r" % args.binary)
+        parser.error(f"Invalid browser binary {args.binary!r}")
     if args.extension is not None:
         for ext in args.extension:
             if not exists(ext):
-                parser.error("Extension %r does not exist" % ext)
+                parser.error(f"Extension {ext!r} does not exist")
     if not isdir(args.logs):
-        parser.error("Log output directory is invalid %r" % args.logs)
-    log_level = log_level_map.get(args.log_level.upper(), None)
-    if log_level is None:
-        parser.error("Invalid log-level %r" % args.log_level)
-    args.log_level = log_level
+        parser.error(f"Log output directory is invalid {args.logs!r}")
+    args.log_level = log_level_map[args.log_level]
     if args.log_limit < 0:
         parser.error("--log-limit must be >= 0")
-    args.log_limit *= 1048576
+    args.log_limit *= 1_048_576
     if args.memory < 0:
         parser.error("--memory must be >= 0")
-    args.memory *= 1048576
+    args.memory *= 1_048_576
     if args.prefs is not None and not isfile(args.prefs):
-        parser.error("Invalid prefs.js file %r" % args.prefs)
+        parser.error(f"Invalid prefs.js file {args.prefs!r}")
 
     return args
 
 
-def main(argv=None):  # pylint: disable=missing-docstring
+def main(argv: Optional[List[str]] = None) -> None:  # pylint: disable=missing-docstring
     args = parse_args(argv)
     # set output verbosity
     if args.log_level == DEBUG:
@@ -255,6 +260,7 @@ def main(argv=None):  # pylint: disable=missing-docstring
             extension=args.extension,
         )
         if args.prefs and isfile(args.prefs):
+            assert ffp.profile is not None
             check_prefs(pathjoin(ffp.profile, "prefs.js"), args.prefs)
         LOG.info("Running Firefox (pid: %d)...", ffp.get_pid())
         while ffp.is_healthy():
