@@ -1,15 +1,17 @@
 #!/usr/bin/env python
 """fake firefox"""
+
 import os
 import platform
 import sys
 import time
-from urllib.request import URLError, urlopen
+from urllib.error import URLError
+from urllib.request import urlopen
 
 EXIT_DELAY = 45
 
 
-def main():  # pylint: disable=missing-docstring
+def main() -> int:  # pylint: disable=missing-docstring
     os_name = platform.system()
     profile = url = None
     while len(sys.argv) > 1:
@@ -23,7 +25,7 @@ def main():  # pylint: disable=missing-docstring
         elif arg == "-profile":
             profile = sys.argv.pop(1)
         else:
-            raise RuntimeError("unknown argument: %s" % arg)
+            raise RuntimeError(f"unknown argument: {arg}")
     if url is None:
         sys.stderr.write("missing url\n")
         return 1
@@ -31,7 +33,7 @@ def main():  # pylint: disable=missing-docstring
     cmd = None
     exit_code = 0
     if profile is not None:
-        with open(os.path.join(profile, "prefs.js"), "r") as prefs_js:
+        with open(os.path.join(profile, "prefs.js")) as prefs_js:
             for line in prefs_js:
                 if line.startswith("user_pref"):
                     pass
@@ -52,10 +54,11 @@ def main():  # pylint: disable=missing-docstring
                 elif line.startswith("#"):
                     pass  # skip comments
                 elif line.strip():
-                    raise RuntimeError("unknown value in prefs.js: %s" % line)
-    # sys.stdout.write('cmd: %s\n' % cmd)
+                    raise RuntimeError(f"unknown value in prefs.js: {line}")
+    # sys.stdout.write(f'cmd: {cmd}\n')
     # sys.stdout.flush()
 
+    assert profile is not None
     if cmd == "invalid_js":
         with open(os.path.join(profile, "Invalidprefs.js"), "w") as prefs_js:
             prefs_js.write("bad!")
@@ -68,18 +71,18 @@ def main():  # pylint: disable=missing-docstring
         except URLError as req_err:
             # can't redirect to file:// from http://
             # pylint: disable=consider-using-with
-            conn = urlopen(req_err.reason.split("'")[1])
+            conn = urlopen(str(req_err.reason).split("'")[1])
         try:
             target_url = conn.geturl()
             if target_url == url:
                 target_url = None
-            sys.stdout.write(conn.read().decode("utf-8"))
+            sys.stdout.write(conn.read().decode())
             sys.stdout.write("\n")
             sys.stdout.flush()
         finally:
             conn.close()
 
-    sys.stdout.write("url: %r\n" % target_url)
+    sys.stdout.write(f"url: {target_url!r}\n")
     sys.stdout.flush()
 
     if cmd == "memory":
@@ -103,11 +106,11 @@ def main():  # pylint: disable=missing-docstring
             sys.stdout.flush()
             sys.stderr.flush()
     elif cmd == "exit_code":
-        sys.stdout.write("exit code test (%d)\n" % exit_code)
+        sys.stdout.write(f"exit code test ({exit_code})\n")
         sys.stdout.flush()
         return exit_code
 
-    sys.stdout.write("running... (sleep %d)\n" % EXIT_DELAY)
+    sys.stdout.write(f"running... (sleep {EXIT_DELAY})\n")
     sys.stdout.flush()
     time.sleep(EXIT_DELAY)  # wait before closing (should be terminated before elapse)
     sys.stdout.write("exiting normally\n")
