@@ -1,22 +1,23 @@
-# coding=utf-8
+# type: ignore
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 # pylint: disable=invalid-name,missing-docstring,protected-access
 """ffpuppet tests"""
-import errno
-import os
-import platform
-import shutil
-import socket
-import stat
-import threading
-import time
-from http.server import BaseHTTPRequestHandler, HTTPServer
-from subprocess import Popen
 
-import pytest
+import os
+from errno import EADDRINUSE
+from http.server import BaseHTTPRequestHandler, HTTPServer
+from pathlib import Path
+from platform import system
+from shutil import rmtree
+from stat import S_IREAD, S_IWRITE
+from subprocess import Popen
+from threading import Thread
+from time import sleep
+
 from psutil import Process
+from pytest import mark, raises
 
 from .bootstrapper import Bootstrapper
 from .core import Debugger, FFPuppet, Reason
@@ -26,13 +27,9 @@ from .exceptions import (
     LaunchError,
     TerminateError,
 )
-from .minidump_parser import MinidumpParser
 
 Bootstrapper.POLL_WAIT = 0.2
-CWD = os.path.realpath(os.path.dirname(__file__))
-TESTFF_BIN = os.path.join(CWD, "resources", "testff.py")
-
-MinidumpParser.MDSW_MAX_STACK = 8
+TESTFF_BIN = str((Path(__file__).parent / "resources" / "testff.py").resolve())
 
 
 class ReqHandler(BaseHTTPRequestHandler):
@@ -43,19 +40,18 @@ class ReqHandler(BaseHTTPRequestHandler):
 
 
 class HTTPTestServer:
-    def __init__(self, handler=None):
-        self._handler = handler if handler is not None else ReqHandler
+    def __init__(self):
+        self._handler = ReqHandler
         while True:
             try:
                 self._httpd = HTTPServer(("127.0.0.1", 0), self._handler)
-            except socket.error as soc_e:
-                if soc_e.errno in (errno.EADDRINUSE, 10013):  # Address already in use
+            except OSError as soc_e:
+                if soc_e.errno in (EADDRINUSE, 10013):
+                    # Address already in use
                     continue
                 raise
             break
-        self._thread = threading.Thread(
-            target=HTTPTestServer._srv_thread, args=(self._httpd,)
-        )
+        self._thread = Thread(target=HTTPTestServer._srv_thread, args=(self._httpd,))
         self._thread.start()
 
     def __enter__(self):
@@ -65,7 +61,7 @@ class HTTPTestServer:
         self.shutdown()
 
     def get_addr(self):
-        return "http://127.0.0.1:%d" % (self._httpd.server_address[1],)
+        return f"http://127.0.0.1:{self._httpd.server_address[1]}"
 
     def shutdown(self):
         if self._httpd is not None:
@@ -81,11 +77,11 @@ class HTTPTestServer:
             httpd.socket.close()
 
 
-@pytest.mark.skipif(platform.system() == "Windows", reason="Unsupported on Windows")
+@mark.skipif(system() == "Windows", reason="Unsupported on Windows")
 def test_ffpuppet_00(tmp_path):
     """test that invalid executables raise the right exception"""
     with FFPuppet() as ffp:
-        with pytest.raises(IOError, match="is not an executable"):
+        with raises(OSError, match="is not an executable"):
             ffp.launch(str(tmp_path))
 
 
@@ -98,13 +94,13 @@ def test_ffpuppet_01():
         assert not ffp.is_running()
         with HTTPTestServer() as srv:
             ffp.launch(TESTFF_BIN, location=srv.get_addr())
-        assert not ffp._checks
-        assert ffp.launches == 1
-        assert not ffp.wait(timeout=0)
-        assert ffp.is_running()
-        assert ffp.is_healthy()
-        assert ffp.reason is None
-        ffp.close()
+            assert not ffp._checks
+            assert ffp.launches == 1
+            assert not ffp.wait(timeout=0)
+            assert ffp.is_running()
+            assert ffp.is_healthy()
+            assert ffp.reason is None
+            ffp.close()
         assert ffp.reason == Reason.CLOSED
         assert ffp._proc is None
         assert not ffp.is_running()
@@ -114,22 +110,21 @@ def test_ffpuppet_01():
 
 def test_ffpuppet_02(mocker):
     """test launch failures"""
+    mocker.patch("ffpuppet.core.Popen", autospec=True)
     fake_bts = mocker.patch("ffpuppet.core.Bootstrapper", autospec=True)
-    bts = mocker.Mock(spec=Bootstrapper, location="")
+    bts = mocker.Mock(spec_set=Bootstrapper, location="")
     fake_bts.return_value = bts
-    proc = mocker.Mock(spec=Popen, pid=0xFFFF)
-    mocker.patch("ffpuppet.core.Popen", autospec=True, return_value=proc)
     # startup crash
     with FFPuppet() as ffp:
         bts.wait.side_effect = BrowserTerminatedError("test")
-        with pytest.raises(BrowserTerminatedError, match="test"):
+        with raises(BrowserTerminatedError, match="test"):
             ffp.launch(TESTFF_BIN)
         assert not ffp.is_running()
         assert ffp.launches == 0
     # startup hang
     with FFPuppet() as ffp:
         bts.wait.side_effect = BrowserTimeoutError("test")
-        with pytest.raises(BrowserTimeoutError, match="test"):
+        with raises(BrowserTimeoutError, match="test"):
             ffp.launch(TESTFF_BIN)
         assert not ffp.is_healthy()
         assert ffp.launches == 0
@@ -178,6 +173,7 @@ def test_ffpuppet_05(mocker):
     with FFPuppet() as ffp:
         assert not ffp.is_running()
         ffp._proc = mocker.Mock(pid=123)
+        assert ffp._proc is not None
         ffp._proc.poll.return_value = None
         assert ffp.is_running()
         ffp._proc.poll.return_value = 0
@@ -190,10 +186,11 @@ def test_ffpuppet_06(mocker):
     """test wait()"""
 
     class StubbedProc(FFPuppet):
-        def close(self, **_):  # pylint: disable=arguments-differ
+        # pylint: disable=arguments-differ
+        def close(self, **_):
             self.reason = Reason.CLOSED
 
-        def launch(self):  # pylint: disable=arguments-differ
+        def launch(self):
             self.reason = None
 
         def get_pid(self):
@@ -213,7 +210,7 @@ def test_ffpuppet_06(mocker):
         assert fake_wait_procs.call_count == 1
         fake_wait_procs.reset_mock()
         # process did not close
-        fake_wait_procs.return_value = ([], [mocker.Mock(spec=Process)])
+        fake_wait_procs.return_value = ([], [mocker.Mock(spec_set=Process)])
         assert not ffp.wait(timeout=10)
         assert fake_wait_procs.call_count == 1
 
@@ -264,7 +261,7 @@ def test_ffpuppet_08(tmp_path):
             for _ in range(100):
                 if not ffp.is_healthy():
                     break
-                time.sleep(0.1)
+                sleep(0.1)
             ffp.close()
         assert ffp.reason == Reason.WORKER
         assert len(ffp.available_logs()) == 3
@@ -285,7 +282,7 @@ def test_ffpuppet_09():
                 ffp.close()
             # call 2x without calling close()
             ffp.launch(TESTFF_BIN, location=srv.get_addr())
-            with pytest.raises(LaunchError, match="Process is already running"):
+            with raises(LaunchError, match="Process is already running"):
                 ffp.launch(TESTFF_BIN)
         assert ffp.launches == 11
         ffp.close()
@@ -304,7 +301,7 @@ def test_ffpuppet_10(tmp_path):
             for _ in range(200):
                 if not ffp.is_healthy():
                     break
-                time.sleep(0.05)
+                sleep(0.05)
             ffp.close()
         assert ffp.reason == Reason.WORKER
         assert len(ffp.available_logs()) == 3
@@ -333,16 +330,16 @@ def test_ffpuppet_12():
             assert ffp.reason is None
             ffp.close()
             ffp.clean_up()
-            with pytest.raises(AssertionError):
+            with raises(AssertionError):
                 ffp.launch(TESTFF_BIN, location=srv.get_addr())
-            with pytest.raises(AssertionError):
+            with raises(AssertionError):
                 ffp.close()
 
 
 def test_ffpuppet_13(mocker):
     """test launching under Xvfb"""
     fake_system = mocker.patch("ffpuppet.core.system", autospec=True)
-    is_linux = platform.system() == "Linux"
+    is_linux = system() == "Linux"
     fake_xvfb = mocker.patch(
         "ffpuppet.core.Xvfb", autospec=is_linux, create=not is_linux
     )
@@ -355,12 +352,12 @@ def test_ffpuppet_13(mocker):
     fake_xvfb.reset_mock()
     # not installed
     fake_xvfb.side_effect = NameError
-    with pytest.raises(EnvironmentError, match="Please install xvfbwrapper"):
+    with raises(OSError, match="Please install xvfbwrapper"):
         FFPuppet(use_xvfb=True)
     assert fake_xvfb.start.call_count == 0
     # unsupported os
     fake_system.return_value = "Windows"
-    with pytest.raises(EnvironmentError, match="Xvfb is only supported on Linux"):
+    with raises(OSError, match="Xvfb is only supported on Linux"):
         FFPuppet(use_xvfb=True)
     assert fake_xvfb.start.call_count == 0
 
@@ -368,7 +365,7 @@ def test_ffpuppet_13(mocker):
 def test_ffpuppet_14(tmp_path):
     """test passing a file and a non existing file to launch() via location"""
     with FFPuppet() as ffp:
-        with pytest.raises(IOError, match="Cannot find"):
+        with raises(OSError, match="Cannot find"):
             ffp.launch(TESTFF_BIN, location="missing.file")
         ffp.close()
         prefs = tmp_path / "prefs.js"
@@ -423,7 +420,7 @@ def test_ffpuppet_16(tmp_path):
     with FFPuppet() as ffp:
         with HTTPTestServer() as srv:
             ffp.launch(TESTFF_BIN, location=srv.get_addr())
-            with pytest.raises(AssertionError):
+            with raises(AssertionError):
                 ffp.save_logs(str(tmp_path / "logs"))
 
 
@@ -458,7 +455,7 @@ def test_ffpuppet_18(tmp_path):
     prefs.write_bytes(b"//fftest_invalid_js\n")
     with FFPuppet() as ffp:
         with HTTPTestServer() as srv:
-            with pytest.raises(LaunchError, match="'.+?' is invalid"):
+            with raises(LaunchError, match="'.+?' is invalid"):
                 ffp.launch(TESTFF_BIN, location=srv.get_addr(), prefs_js=str(prefs))
 
 
@@ -468,9 +465,13 @@ def test_ffpuppet_19():
         assert ffp.log_length("INVALID") is None
         assert ffp.log_length("stderr") is None
         ffp.launch(TESTFF_BIN)
-        assert ffp.log_length("stderr") > 0
+        ll = ffp.log_length("stderr")
+        assert ll is not None
+        assert ll > 0
         ffp.close()
-        assert ffp.log_length("stderr") > 0
+        ll = ffp.log_length("stderr")
+        assert ll is not None
+        assert ll > 0
         ffp.clean_up()
         # verify clean_up() removed the logs
         assert ffp.log_length("stderr") is None
@@ -507,7 +508,7 @@ def test_ffpuppet_21(tmp_path):
         for _ in range(100):
             if not ffp.is_healthy():
                 break
-            time.sleep(0.1)
+            sleep(0.1)
         ffp.close()
         assert ffp.reason == Reason.WORKER
         logs = tmp_path / "logs"
@@ -526,6 +527,7 @@ def test_ffpuppet_22(tmp_path):
     test_logs = list()
     with FFPuppet() as ffp:
         ffp.launch(TESTFF_BIN)
+        assert ffp._logs.working_path is not None
         asan_prefix = os.path.join(ffp._logs.working_path, ffp._logs.PREFIX_SAN)
         for i in range(4):
             test_logs.append(".".join([asan_prefix, str(i)]))
@@ -585,7 +587,7 @@ def test_ffpuppet_23(mocker, tmp_path):
     # pylint: disable=unused-argument
     def _fake_process_minidumps(dmps, _, add_log, working_path=None):
         for num, _ in enumerate(x for x in os.listdir(dmps) if x.endswith(".dmp")):
-            lfp = add_log("minidump_%02d" % (num + 1,))
+            lfp = add_log(f"minidump_{num + 1:02}")
             lfp.write(b"test")
 
     mocker.patch("ffpuppet.core.process_minidumps", side_effect=_fake_process_minidumps)
@@ -595,14 +597,12 @@ def test_ffpuppet_23(mocker, tmp_path):
     with FFPuppet(use_profile=str(profile)) as ffp:
         ffp.launch(TESTFF_BIN)
         ffp._bin_path = ffp.profile
+        assert ffp._bin_path is not None
         # create "test.dmp" files
-        md_path = os.path.join(ffp._bin_path, "minidumps")
-        with open(os.path.join(md_path, "test1.dmp"), "w") as out_fp:
-            out_fp.write("1a\n1b")
-        with open(os.path.join(md_path, "test2.dmp"), "w") as out_fp:
-            out_fp.write("2a\n2b")
-        with open(os.path.join(md_path, "test3.dmp"), "w") as out_fp:
-            out_fp.write("3a\n3b")
+        md_path = Path(ffp._bin_path) / "minidumps"
+        (md_path / "test1.dmp").write_text("1a\n1b")
+        (md_path / "test2.dmp").write_text("2a\n2b")
+        (md_path / "test3.dmp").write_text("3a\n3b")
         assert not ffp.is_healthy()
         ffp.close()
         logs = tmp_path / "logs"
@@ -641,10 +641,10 @@ def test_ffpuppet_25(tmp_path):
     # - just create a puppet, write a readonly file in its profile, then call close()
     with FFPuppet() as ffp:
         ffp.launch(TESTFF_BIN)
-        ro_file = os.path.join(ffp.profile, "read-only-test.txt")
-        with open(ro_file, "w"):
-            pass
-        os.chmod(ro_file, stat.S_IREAD)
+        assert ffp.profile is not None
+        ro_file = Path(ffp.profile) / "read-only-test.txt"
+        ro_file.touch()
+        ro_file.chmod(S_IREAD)
         ffp.close()
         assert not os.path.isfile(ro_file)
         ffp.clean_up()
@@ -653,9 +653,10 @@ def test_ffpuppet_25(tmp_path):
     profile.mkdir()
     ro_file = profile / "read-only.txt"
     ro_file.touch()
-    os.chmod(str(ro_file), stat.S_IREAD)
+    ro_file.chmod(S_IREAD)
     with FFPuppet(use_profile=str(profile)) as ffp:
         ffp.launch(TESTFF_BIN)
+        assert ffp.profile is not None
         prof_path = ffp.profile
         assert os.path.isdir(prof_path)
         ffp.close()
@@ -666,14 +667,15 @@ def test_ffpuppet_26(tmp_path):
     """test using a readonly prefs.js and extension"""
     prefs = tmp_path / "prefs.js"
     prefs.touch()
-    os.chmod(str(prefs), stat.S_IREAD)
+    prefs.chmod(S_IREAD)
     ext = tmp_path / "ext.xpi"
     ext.touch()
-    os.chmod(str(ext), stat.S_IREAD)
+    ext.chmod(S_IREAD)
     with FFPuppet() as ffp:
         ffp.launch(TESTFF_BIN, extension=str(ext), prefs_js=str(prefs))
         prof_path = ffp.profile
         ffp.close()
+        assert prof_path is not None
         assert not os.path.isdir(prof_path)
 
 
@@ -684,64 +686,62 @@ def test_ffpuppet_27(mocker, tmp_path):
     )
 
     class StubbedLaunch(FFPuppet):
-        def launch(self):  # pylint: disable=arguments-differ
+        # pylint: disable=arguments-differ
+        def launch(self):
             profile = tmp_path / "profile"
             profile.mkdir()
             (profile / "minidumps").mkdir()
             self.profile = str(profile)
 
         def close(self, force_close=False):
+            assert self.profile is not None
             if os.path.isdir(self.profile):
-                shutil.rmtree(self.profile)
+                rmtree(self.profile)
             self.profile = None
 
-    is_linux = platform.system() == "Linux"
+    is_linux = system() == "Linux"
+    # only check Valgrind logs on Linux
     debugger = Debugger.VALGRIND if is_linux else Debugger.NONE
     with StubbedLaunch(debugger=debugger) as ffp:
         assert ffp._dbg == debugger
         ffp.launch()
         assert not any(ffp._crashreports())
         # benign sanitizer warnings
-        ign_log = "%s.1" % (ffp._logs.PREFIX_SAN,)
-        with open(os.path.join(ffp._logs.working_path, ign_log), "w") as ofp:
-            ofp.write(
-                "==123==WARNING: Symbolizer buffer too small\n\n"
-                "==123==WARNING: Symbolizer buffer too small\n\n"
-                "==123==WARNING: AddressSanitizer failed to allocate 0xFFFFFF bytes\n"
-                "==123==AddressSanitizer: soft rss limit exhausted (5000Mb vs 5026Mb)\n"
-            )
+        assert ffp._logs.working_path is not None
+        ign_log = Path(ffp._logs.working_path) / (f"{ffp._logs.PREFIX_SAN}.1")
+        ign_log.write_text(
+            "==123==WARNING: Symbolizer buffer too small\n\n"
+            "==123==WARNING: Symbolizer buffer too small\n\n"
+            "==123==WARNING: AddressSanitizer failed to allocate 0xFFFFFF bytes\n"
+            "==123==AddressSanitizer: soft rss limit exhausted (5000Mb vs 5026Mb)\n"
+        )
         assert any(ffp._crashreports(skip_benign=False))
         # valid sanitizer log
-        san_log = "%s.2" % (ffp._logs.PREFIX_SAN,)
-        with open(os.path.join(ffp._logs.working_path, san_log), "w") as ofp:
-            ofp.write("test\n")
+        san_log = Path(ffp._logs.working_path) / (f"{ffp._logs.PREFIX_SAN}.2")
+        san_log.write_text("test\n")
         # valid Valgrind log - with error
-        vg1_log = "%s.1" % (ffp._logs.PREFIX_VALGRIND,)
-        with open(os.path.join(ffp._logs.working_path, vg1_log), "w") as ofp:
-            ofp.write("test\n")
+        vg1_log = Path(ffp._logs.working_path) / (f"{ffp._logs.PREFIX_VALGRIND}.1")
+        vg1_log.write_text("test\n")
         # valid Valgrind log - without error
-        vg2_log = "%s.2" % (ffp._logs.PREFIX_VALGRIND,)
-        with open(os.path.join(ffp._logs.working_path, vg2_log), "w") as ofp:
-            pass
+        (Path(ffp._logs.working_path) / (f"{ffp._logs.PREFIX_VALGRIND}.2")).touch()
         # nothing interesting
-        with open(os.path.join(ffp._logs.working_path, "junk.log"), "w") as ofp:
-            ofp.write("test\n")
+        (Path(ffp._logs.working_path) / "junk.log").write_text("test\n")
         # valid minidump
-        with open(os.path.join(ffp.profile, "minidumps", "test.dmp"), "w") as ofp:
-            ofp.write("test\n")
+        assert ffp.profile is not None
+        (Path(ffp.profile) / "minidumps" / "test.dmp").write_text("test\n")
         # nothing interesting
-        with open(os.path.join(ffp.profile, "minidumps", "test.junk"), "w") as ofp:
-            pass
+        (Path(ffp.profile) / "minidumps" / "test.junk").write_text("\n")
         assert not ffp._logs.watching
-        # NOTE: Valgrind logs are only checked on Linux
         assert len(list(ffp._crashreports())) == (3 if is_linux else 2)
         assert ffp._logs.watching
         assert len(list(ffp._crashreports(skip_md=True))) == (2 if is_linux else 1)
-        # fail to open and scan sanitizer file
-        ffp._logs.watching.clear()
-        mocker.patch("ffpuppet.core.open", side_effect=OSError)
-        assert len(list(ffp._crashreports())) == (4 if is_linux else 3)
-        assert not ffp._logs.watching
+        if system() != "Windows":
+            # fail to open (for read) and scan sanitizer file
+            # not tested on Windows because chmod() does not work
+            ffp._logs.watching.clear()
+            ign_log.chmod(S_IWRITE)
+            assert len(list(ffp._crashreports())) == (4 if is_linux else 3)
+            assert not ffp._logs.watching
 
 
 def test_ffpuppet_28(tmp_path):
@@ -772,7 +772,7 @@ def test_ffpuppet_28(tmp_path):
         ffp._dbg = Debugger.VALGRIND
         try:
             os.environ["VALGRIND_SUP_PATH"] = "blah"
-            with pytest.raises(IOError):
+            with raises(OSError):
                 ffp.build_launch_cmd("bin_path")
             supp = tmp_path / "suppressions.txt"
             supp.touch()
@@ -810,13 +810,13 @@ def test_ffpuppet_30(mocker):
     fake_chkout.reset_mock()
     # gdb - not installed
     fake_chkout.side_effect = OSError
-    with pytest.raises(EnvironmentError, match="Please install GDB"):
+    with raises(OSError, match="Please install GDB"):
         FFPuppet._dbg_sanity_check(Debugger.GDB)
     fake_chkout.reset_mock()
     fake_chkout.side_effect = None
     # gdb - unsupported OS
     fake_system.return_value = "Windows"
-    with pytest.raises(EnvironmentError, match="GDB is only supported on Linux"):
+    with raises(OSError, match="GDB is only supported on Linux"):
         FFPuppet._dbg_sanity_check(Debugger.GDB)
     # rr - success
     fake_system.return_value = "Linux"
@@ -825,42 +825,45 @@ def test_ffpuppet_30(mocker):
     fake_chkout.reset_mock()
     # rr - not installed
     fake_chkout.side_effect = OSError
-    with pytest.raises(EnvironmentError, match="Please install rr"):
+    with raises(OSError, match="Please install rr"):
         FFPuppet._dbg_sanity_check(Debugger.RR)
     fake_chkout.reset_mock()
     fake_chkout.side_effect = None
     # rr - unsupported OS
     fake_system.return_value = "Windows"
-    with pytest.raises(EnvironmentError, match="rr is only supported on Linux"):
+    with raises(OSError, match="rr is only supported on Linux"):
         FFPuppet._dbg_sanity_check(Debugger.RR)
     # valgrind - success
     fake_system.return_value = "Linux"
-    fake_chkout.return_value = b"valgrind-%0.2f" % (FFPuppet.VALGRIND_MIN_VERSION)
+    fake_chkout.return_value = f"valgrind-{FFPuppet.VALGRIND_MIN_VERSION:.2f}".encode()
     FFPuppet._dbg_sanity_check(Debugger.VALGRIND)
     assert fake_chkout.call_count == 1
     fake_chkout.reset_mock()
     # valgrind - old version
     fake_system.return_value = "Linux"
     fake_chkout.return_value = b"valgrind-0.1"
-    with pytest.raises(EnvironmentError, match=r"Valgrind >= \d+\.\d+ is required"):
+    with raises(OSError, match=r"Valgrind >= \d+\.\d+ is required"):
         FFPuppet._dbg_sanity_check(Debugger.VALGRIND)
     assert fake_chkout.call_count == 1
     fake_chkout.reset_mock()
     # valgrind - not installed
     fake_chkout.side_effect = OSError
-    with pytest.raises(EnvironmentError, match="Please install Valgrind"):
+    with raises(OSError, match="Please install Valgrind"):
         FFPuppet._dbg_sanity_check(Debugger.VALGRIND)
     fake_chkout.reset_mock()
     fake_chkout.side_effect = None
     # valgrind - unsupported OS
     fake_system.return_value = "Windows"
-    with pytest.raises(EnvironmentError, match="Valgrind is only supported on Linux"):
+    with raises(OSError, match="Valgrind is only supported on Linux"):
         FFPuppet._dbg_sanity_check(Debugger.VALGRIND)
 
 
 def test_ffpuppet_31(mocker):
     """test _terminate()"""
-    procs = [mocker.Mock(spec=Process, pid=123), mocker.Mock(spec=Process, pid=124)]
+    procs = [
+        mocker.Mock(spec_set=Process, pid=123),
+        mocker.Mock(spec_set=Process, pid=124),
+    ]
     mocker.patch("ffpuppet.core.get_processes", autospec=True, return_value=procs)
     fake_wait_procs = mocker.patch("ffpuppet.core.wait_procs", autospec=True)
     # successful call to terminate (parent process only)
@@ -890,7 +893,7 @@ def test_ffpuppet_31(mocker):
     # failed call to kill
     fake_wait_procs.return_value = ([], procs)
     fake_wait_procs.side_effect = None
-    with pytest.raises(TerminateError):
+    with raises(TerminateError):
         FFPuppet._terminate(1234)
 
 
@@ -898,7 +901,8 @@ def test_ffpuppet_32(mocker, tmp_path):
     """test FFPuppet.close() setting reason"""
 
     class StubbedProc(FFPuppet):
-        def launch(self):  # pylint: disable=arguments-differ
+        # pylint: disable=arguments-differ
+        def launch(self):
             self.reason = None
             self._bin_path = str(tmp_path)
             self._logs.reset()
@@ -946,7 +950,7 @@ def test_ffpuppet_32(mocker, tmp_path):
     with StubbedProc() as ffp:
         ffp.launch()
         ffp._proc.poll.return_value = None
-        fake_reports.return_value = (str(tmp_path / "fake_report1"),)
+        fake_reports.return_value = (tmp_path / "fake_report1",)
         fake_wait_files.return_value = False
         ffp.close()
         assert ffp._proc is None
@@ -959,23 +963,11 @@ def test_ffpuppet_32(mocker, tmp_path):
         ffp._proc.poll.return_value = None
         fake_reports.return_value = None
         fake_reports.side_effect = (
-            (str(tmp_path / "fake_report1"),),
-            (
-                str(tmp_path / "fake_report1"),
-                str(tmp_path / "fake_report2"),
-            ),
-            (
-                str(tmp_path / "fake_report1"),
-                str(tmp_path / "fake_report2"),
-            ),
-            (
-                str(tmp_path / "fake_report1"),
-                str(tmp_path / "fake_report2"),
-            ),
-            (
-                str(tmp_path / "fake_report1"),
-                str(tmp_path / "fake_report2"),
-            ),
+            (tmp_path / "fake_report1",),
+            (tmp_path / "fake_report1", tmp_path / "fake_report2"),
+            (tmp_path / "fake_report1", tmp_path / "fake_report2"),
+            (tmp_path / "fake_report1", tmp_path / "fake_report2"),
+            (tmp_path / "fake_report1", tmp_path / "fake_report2"),
         )
         fake_wait_files.return_value = True
         ffp.close()
@@ -988,7 +980,8 @@ def test_ffpuppet_33():
     """test ignoring benign sanitizer logs"""
     with FFPuppet() as ffp:
         ffp.launch(TESTFF_BIN)
-        san_log = "%s.1" % os.path.join(ffp._logs.working_path, ffp._logs.PREFIX_SAN)
+        assert ffp._logs.working_path is not None
+        san_log = f"{os.path.join(ffp._logs.working_path, ffp._logs.PREFIX_SAN)}.1"
         assert not ffp._logs.watching
         # ignore benign ASan warning
         with open(san_log, "w") as log_fp:
