@@ -1,8 +1,8 @@
-# coding=utf-8
-"""ffpuppet main.py tests"""
+# type: ignore
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
+"""ffpuppet main.py tests"""
 
 from pytest import mark, raises
 
@@ -83,9 +83,6 @@ def test_parse_args_01(tmp_path):
     (tmp_path / "junk.log").touch()
     with raises(SystemExit):
         parse_args([str(fake_bin), "--logs", "/missing/path/"])
-    # invalid log level
-    with raises(SystemExit):
-        parse_args([str(fake_bin), "--log-level", "bad"])
     # success
     assert parse_args([str(fake_bin)])
 
